#!/usr/bin/env python3
"""Update the copy of ptext in Pygame Zero.

ptext is not yet packaged on PyPI; this script exists to mirror it into the
local repository to make it easily installable.

"""
import json
import base64
import subprocess
from urllib.parse import urljoin
from urllib.request import build_opener


FILE = 'ptext.py'
DEST = 'pgzero/ptext.py'
REPO_URL = 'https://api.github.com/repos/cosmologicon/pygame-text/'
HEADER = '''"""pygame-text - high-level text rendering with Pygame.

This module is directly copied from

    https://github.com/cosmologicon/pygame-text

at revision {sha}
and used under CC0.

"""
'''


# Customise the opener here if you need to
opener = build_opener()


def read_json(url):
    """Download and decode a JSON resource from the given URL."""
    resp = opener.open(url)
    charset = resp.headers.get_content_charset()
    data = resp.read().decode(charset)
    return json.loads(data)


def get_tree():
    """Download the repository tree, returning a decoded JSON structure."""
    print('Downloading repository tree...')
    url = urljoin(REPO_URL, 'git/trees/HEAD')
    return read_json(url)


def get_file(file):
    """Download the tree state and named file.

    Return a tuple of the current repo version hash and the file's data.

    """
    tree = get_tree()
    for f in tree['tree']:
        if f['path'] == file:
            break
    else:
        raise ValueError("Could not find ptext module to download.")

    url = f['url']
    print('Downloading', file, 'module...')
    blob = read_json(url)
    data = base64.b64decode(blob['content']).decode('utf8')
    return tree['sha'], data


def update_local():
    """Download a new copy of the file and write it to DEST.

    Include a header based on the template HEADER.

    """
    sha, data = get_file(FILE)
    header = HEADER.format(sha=sha)
    with open(DEST, 'w', encoding='utf8') as f:
        f.write(header + data)
    print("Updated", FILE, "to revision", sha[:7])
    autopep8()


def autopep8():
    """Use autopep8 to fix formatting problems."""
    print("Running autopep8")
    subprocess.check_call(['autopep8', '-i', DEST])


if __name__ == '__main__':
    update_local()



import sys
import unittest
import pygame
import pygame.image
import numpy as np
from pathlib import Path

from pgzero.screen import Screen
from pgzero.loaders import set_root, images

pygame.init()
surf = pygame.display.set_mode((200, 100))


class ScreenTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        """Initialise the display and set loaders to target the current dir."""
        pygame.init()
        cls.surf = pygame.display.set_mode((200, 100))
        set_root(__file__)

    @classmethod
    def tearDownClass(cls):
        """Shut down the display."""
        pygame.display.quit()

    def setUp(self):
        self.screen = Screen(self.surf)
        self.screen.clear()

    def assertImagesAlmostEqual(self, computed, expected):
        """Check that 2 images are approximately equal."""
        comp_surf = pygame.surfarray.array3d(computed)
        exp_surf = pygame.surfarray.array3d(expected)

        if np.allclose(comp_surf, exp_surf, atol=7):
            return

        name = sys._getframe(1).f_code.co_name
        tmpdir = Path(__file__).parent / 'failed-image'
        tmpdir.mkdir(exist_ok=True)
        pygame.image.save(
            computed,
            str(tmpdir / '{}-computed.png'.format(name))
        )
        pygame.image.save(
            expected,
            str(tmpdir / '{}-expected.png'.format(name))
        )

        raise AssertionError(
            "Images differ; saved comparison images to {}".format(tmpdir)
        )

    def test_blit_surf(self):
        """We can blit a surface to the screen."""
        self.screen.blit(images.alien, (0, 0))
        self.assertImagesAlmostEqual(surf, images.expected_alien_blit)

    def test_blit_name(self):
        """screen.blit() accepts an image name instead of a Surface."""
        self.screen.blit('alien', (0, 0))
        self.assertImagesAlmostEqual(surf, images.expected_alien_blit)


if __name__ == '__main__':
    unittest.main()



import random, math

"""
Pi Lander
 * A basic Lunar Lander style game in Pygame Zero
 * Run with 'pgzrun pi_lander.py', control with the LEFT, RIGHT and UP arrow keys
 * Author Tim Martin: www.Tim-Martin.co.uk
 * Licence: Creative Commons Attribution-ShareAlike 4.0 International
 * http://creativecommons.org/licenses/by-sa/4.0/
"""

WIDTH = 800 # Screen width
HEIGHT = 600 # Screen height

class LandingSpotClass:
    """ Each instance defines a landing spot by where it starts, how big it is and how many points it's worth """
    landing_spot_sizes = ["small", "medium", "large"]
    def __init__(self, starting_step):
        self.starting = starting_step
        random_size = random.choice(LandingSpotClass.landing_spot_sizes) # And randomly choose size
        if random_size == "small":
            self.size = 4
            self.bonus = 8
        elif random_size == "medium":
            self.size = 10
            self.bonus = 4
        else: # Large
            self.size = 20
            self.bonus = 2
    def get_within_landing_spot(self, step):
        if (step >= self.starting) and (step < self.starting + self.size):
            return True
        return False

class LandscapeClass:
    """ Stores and generates the landscape, landing spots and star field """
    step_size = 3 # Landscape is broken down into steps. Define number of pixels on the x axis per step.
    world_steps = int(WIDTH/step_size) # How many steps can we fit horizontally on the screen
    small_height_change = 3 # Controls how bumpy the landscape is
    large_height_change = 10 # Controls how steep the landscape is
    features = ["mountain","valley","field"] # What features to generate
    n_stars = 30 # How many stars to put in the background
    n_spots = 4 # Max number of landing spots to generate
    def __init__(self):
        self.world_height = [] # Holds the height of the landscape at each step
        self.star_locations = [] # Holds the x and y location of the stars
        self.landing_spots = [] # Holds the landing spots
    def get_within_landing_spot(self, step):
        """ Calculate if a given step is within any of the landing spots """
        for spot in self.landing_spots:
            if spot.get_within_landing_spot(step) == True:
                return True
        return False
    def get_landing_spot_bonus(self, step):
        for spot in self.landing_spots:
            if spot.get_within_landing_spot(step) == True:
                return spot.bonus
        return 0
    def reset(self):
        """ Generates a new landscape """
        # First: Choose which steps of the landscape will be landing spots
        del self.landing_spots[:] # Delete any previous LandingSpotClass objects
        next_spot_start = 0
        # Move from left to right adding new landing spots until either
        # n_spots spots have been placed or we run out of space in the world
        while len(self.landing_spots) < LandscapeClass.n_spots and next_spot_start < LandscapeClass.world_steps:
            next_spot_start += random.randint(10, 50) # Randomly choose location to start landing spot
            new_landing_spot = LandingSpotClass(next_spot_start) # Make a new landing object at this spot
            self.landing_spots.append( new_landing_spot ) # And store it in our list
            next_spot_start += new_landing_spot.size # Then take into account its size before choosing the next
        # Second: Randomise the world map
        del self.world_height[:] # Clear any previous world height data
        feature_steps = 0 # Keep track of how many steps we are into a feature
        self.world_height.append(random.randint(300, 500)) # Start the landscape between 300 and 500 pixels down
        for step in range(1, LandscapeClass.world_steps):
            # If feature_step is zero, we need to choose a new feature and how long it goes on for
            if feature_steps == 0:
                feature_steps = random.randint(25, 75)
                current_feature = random.choice(LandscapeClass.features)
            # Generate the world by setting the range of random numbers, must be flat if in a landing spot
            if self.get_within_landing_spot(step) == True:
                max_up = 0 # Flat
                max_down = 0 # Flat
            elif current_feature == "mountain":
                max_up = LandscapeClass.small_height_change
                max_down = -LandscapeClass.large_height_change
            elif current_feature == "valley":
                max_up = LandscapeClass.large_height_change
                max_down = -LandscapeClass.small_height_change
            elif current_feature == "field":
                max_up = LandscapeClass.small_height_change
                max_down = -LandscapeClass.small_height_change
            # Generate the next piece of the landscape
            current_height = self.world_height[step-1]
            next_height = current_height + random.randint(max_down, max_up)
            self.world_height.append(next_height)
            feature_steps -= 1
            # Stop mountains getting too high, or valleys too low
            if next_height > 570:
                current_feature = "mountain" # Too low! Force a mountain
            elif next_height < 200:
                current_feature = "valley" # Too high! Force a valley
        # Third: Randomise the star field
        del self.star_locations[:]
        for star in range(0, LandscapeClass.n_stars):
            star_step = random.randint(0, LandscapeClass.world_steps-1)
            star_x = star_step * LandscapeClass.step_size
            star_y = random.randint( 0, self.world_height[star_step] ) # Keep the stars above the landscape
            self.star_locations.append( (star_x, star_y) )

class ShipClass:
    """ Holds the state of the player's ship and handles movement """
    max_fuel = 1000 # How much fuel the player starts with
    booster_power = 0.05 # Power of the ship's thrusters
    rotate_speed = 10 # How fast the ship rotates in degrees per frame
    gravity = [0., 0.01] # Strength of gravity in the x and y directions
    def __init__(self):
        """ Create the variables which will describe the players ship """
        self.angle = 0 # The angle the ship is facing 0 - 360 degrees
        self.altitude = 0 # The number of pixels the ship is above the ground
        self.booster = False # True if the player is firing their booster
        self.fuel = 0 # Amount of fuel remaining
        self.position = [0,0] # The x and y coordinates of the players ship
        self.velocity = [0,0] # The x and y velocity of the players ship
        self.acceleration = [0,0] # The x and y acceleration of the players ship
    def reset(self):
        """ Set the ships position, velocity and angle to their new-game values """
        self.position = [750., 100.] # Always start at the same spot
        self.velocity = [ -random.random(), random.random() ] # But with some initial speed
        self.acceleration = [0., 0.] # No initial acceleration (except gravity of course)
        self.angle = random.randint(0, 360) # And pointing in a random direction
        self.fuel = ShipClass.max_fuel # Fill up fuel tanks
    def rotate(self, direction):
        """ Rotate the players ship and keep the angle within the range 0 - 360 degrees """
        if direction == "left":
            self.angle -= ShipClass.rotate_speed
        elif direction == "right":
            self.angle += ShipClass.rotate_speed
        if self.angle > 360: # Remember than adding or subtracting 360 degrees does not change the angle
            self.angle -= 360
        elif self.angle < 0:
            self.angle += 360
    def booster_on(self):
        """ When booster is firing we accelerate in the opposite direction, 180 degrees, from the way the ship is facing """
        self.booster = True
        self.acceleration[0] = ShipClass.booster_power * math.sin( math.radians(self.angle + 180) )
        self.acceleration[1] = ShipClass.booster_power * math.cos( math.radians(self.angle + 180) )
        self.fuel -= 2;
    def booster_off(self):
        """ When the booster is not firing we do not accelerate """
        self.booster = False
        self.acceleration[0] = 0.
        self.acceleration[1] = 0.
    def update_physics(self):
        """ Update ship physics in X and Y, apply acceleration (and gravity) to the velocity and velocity to the position """
        for axis in range(0,2):
            self.velocity[axis] += ShipClass.gravity[axis]
            self.velocity[axis] += self.acceleration[axis]
            self.position[axis] += self.velocity[axis]
        # Update player altitude. Note that (LanscapeClass.step_size * 3) is the length of the ship's legs
        ship_step = int(self.position[0]/LandscapeClass.step_size)
        if ship_step < LandscapeClass.world_steps:
            self.altitude = game.landscape.world_height[ship_step] - self.position[1] - (LandscapeClass.step_size * 3)
    def get_out_of_bounds(self):
        """ Check if the player has hit the ground or gone off the sides """
        if self.altitude <= 0 or self.position[0] <= 0 or self.position[0] >= WIDTH:
            return True
        return False

class GameClass:
    """ Holds main game data, including the ship and landscape objects. Checks for game-over """
    def __init__(self):
        self.time = 0. # Time spent playing in seconds
        self.score = 0 # Player's score
        self.game_speed = 30 # How fast the game should run in frames per second
        self.time_elapsed = 0. # Time since the last frame was changed
        self.blink = True # True if blinking text is to be shown
        self.n_frames = 0 # Number of frames processed
        self.game_on = False # True if the game is being played
        self.game_message = "PI   LANDER\nPRESS SPACE TO START" # Start of game message
        self.ship = ShipClass() # Make a object of the ShipClass type
        self.landscape = LandscapeClass()
        self.reset() # Start the game with a fresh landscape and ship
    def reset(self):
        self.time = 0.
        self.ship.reset()
        self.landscape.reset()
    def check_game_over(self):
        """ Check if the game is over and update the game state if so """
        if self.ship.get_out_of_bounds() == False:
            return # Game is not over
        self.game_on = False # Game has finished. But did we win or loose?
        # Check if the player looses. This is if the ship's angle is > 20 degrees
        # the ship is not over a landing site, is moving too fast or is off the side of the screen
        ship_step = int(self.ship.position[0]/LandscapeClass.step_size)
        if self.ship.position[0] <= 0 \
           or self.ship.position[0] >= WIDTH \
           or self.landscape.get_within_landing_spot(ship_step) == False \
           or abs(self.ship.velocity[0]) > .5 \
           or abs(self.ship.velocity[1]) > .5 \
           or (self.ship.angle > 20 and self.ship.angle < 340):
            self.game_message = "YOU JUST DESTROYED A 100 MEGABUCK LANDER\n\nLOOSE 250 POINTS\n\nPRESS SPACE TO RESTART"
            self.score -= 250
        else: # If the player has won! Update their score based on the amount of remaining fuel and the landing bonus
            points = self.ship.fuel / 10
            points *= self.landscape.get_landing_spot_bonus(ship_step)
            self.score += points
            self.game_message = "CONGRATULATIONS\nTHAT WAS A GREAT LANDING!\n\n" + str(round(points)) + " POINTS\n\nPRESS SPACE TO RESTART"

# Create the game object
game = GameClass()

def draw():
    """
    Draw the game window on the screen in the following order:
    start message, mountain range, bonus points, stars, statistics, player's ship
    """
    screen.fill("black")
    size = LandscapeClass.step_size

    if game.game_on == False:
        screen.draw.text(game.game_message, center=(WIDTH/2, HEIGHT/5), align="center")

    # Get the x and y coordinates of each step of the landscape and draw it as a straight line
    for step in range(0, game.landscape.world_steps - 1):
        x_start = size * step
        x_end   = size * (step + 1)
        y_start = game.landscape.world_height[step]
        y_end   = game.landscape.world_height[step + 1]
        screen.draw.line( (x_start, y_start), (x_end, y_end), "white" )
        # Every second we flash the landing spots with a thicker line by drawing a narrow rectangle
        if (game.blink == True or game.game_on == False) and game.landscape.get_within_landing_spot(step) == True:
            screen.draw.filled_rect( Rect(x_start-size, y_start-1, size, 3), "white" )

    # Draw the bonus point notifier
    if game.blink == True or game.game_on == False:
        for spot in game.landscape.landing_spots:
            x_text = spot.starting * size
            y_text = game.landscape.world_height[ spot.starting ] + 10 # The extra 10 pixels puts the text below the landscape
            screen.draw.text(str(spot.bonus) + "x", (x_text,y_text), color="white")

    # Draw the stars
    for star in game.landscape.star_locations:
        screen.draw.line( star, star, "white" )

    # Draw the stats
    screen.draw.text("SCORE: " + str(round(game.score)), (10,10), color="white", background="black")
    screen.draw.text("TIME: " + str(round(game.time)), (10,25), color="white", background="black")
    screen.draw.text("FUEL: " + str(game.ship.fuel), (10,40), color="white", background="black")
    screen.draw.text("ALTITUDE: " + str(round(game.ship.altitude)), (WIDTH-230,10), color="white", background="black")
    screen.draw.text("HORIZONTAL SPEED: {0:.2f}".format(game.ship.velocity[0]), (WIDTH-230,25), color="white", background="black")
    screen.draw.text("VERTICAL SPEED: {0:.2f}".format(-game.ship.velocity[1]), (WIDTH-230,40), color="white", background="black")

    screen.draw.circle( game.ship.position, size*2, "yellow" ) # Draw the player
    # Use sin and cosine functions to draw the ship legs and booster at the correct angle
    # Requires the values in radians (0 to 2*pi) rather than in degrees (0 to 360)
    sin_angle = math.sin( math.radians(game.ship.angle - 45) ) # Legs are drawn 45 degrees either side of the ship's angle
    cos_angle = math.cos( math.radians(game.ship.angle - 45) )
    screen.draw.line( game.ship.position, (game.ship.position[0] + (sin_angle*size*3), game.ship.position[1] + (cos_angle*size*3)), "yellow" )
    sin_angle = math.sin( math.radians(game.ship.angle + 45) )
    cos_angle = math.cos( math.radians(game.ship.angle + 45) )
    screen.draw.line( game.ship.position, (game.ship.position[0] + (sin_angle*size*3), game.ship.position[1] + (cos_angle*size*3)), "yellow" )
    if game.ship.booster == True:
        sin_angle = math.sin( math.radians(game.ship.angle) ) # Booster is drawn at the same angle as the ship, just under it
        cos_angle = math.cos( math.radians(game.ship.angle) )
        screen.draw.filled_circle( (game.ship.position[0] + (sin_angle*size*3), game.ship.position[1] + (cos_angle*size*3)), size, "orange" )

def update(detlatime):
    """ Updates the game physics 30 times every second  """
    game.time_elapsed += detlatime
    if game.time_elapsed < 1./game.game_speed:
        return # A 30th of a second has not passed yet
    game.time_elapsed -= 1./game.game_speed

    # New frame - do all the simulations
    game.n_frames += 1
    if game.n_frames % game.game_speed == 0: # If n_frames is an exact multiple of the game FPS: so once per second
        game.blink = not game.blink # Invert blink so True becomes False or False becomes True

    # Start the game if the player presses space when the game is not on
    if keyboard.space and game.game_on == False:
        game.game_on = True
        game.reset()
    elif game.game_on == False:
        return

    # If the game is on, update the movement and the physics
    if keyboard.left: # Change space ship rotation
        game.ship.rotate("left")
    elif keyboard.right:
        game.ship.rotate("right")

    if keyboard.up and game.ship.fuel > 0: # Fire boosters if the player has enough fuel
        game.ship.booster_on()
    else:
        game.ship.booster_off()

    game.time += detlatime
    game.ship.update_physics()
    game.check_game_over()



'''
The MIT License (MIT)

Copyright (c) 2015 David Bern

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

'''
import random

def grid_list(block_size):
    '''
    Generates a gridlist used shape class
    '''
    grids = []
    for y in range(-5, 1):
        row = []
        for x in range(0, 6):
            row.append((x * block_size, y * block_size))
        grids.append(row)

    return grids

class GameOver(Exception):
    '''
    Exception that is raised when a brick causes a game over state.
    '''
    pass


class Shape(object):
    '''
    This will make the foundation for a Shape that the others shapes
    will inherit.
    '''
    def __init__(self, x_pos, size, color):
        '''
        Inits the shape to start at y_pos and every block is of size in size.
        OShape consists of 4 blocks, each block is size wide and high.
        '''
        # the widths and heights is determined by the shape and block size.
        self.block_size = size
        self._width = size
        self._height = size

        # Rows of blocks.
        # _block_init() sets allocates all the rectangles and resets the shape
        # to start position.
        self._x = x_pos - (self.block_size * 3)
        self._y = 0
        self.piece = []

        self.grid = grid_list(self.block_size)

        self.color = color

    def shape_grid_update(self):
        '''
        Update shapes position with the given _x and _y value.
        To be used after move or rotation is performed on the shape.
        '''
        for row_n, row in enumerate(self.piece):
            for element_n, element in enumerate(row):
                if element is not None:
                    start_x, start_y = self.grid[row_n][element_n]
                    self.piece[row_n][element_n].x = start_x + self._x
                    self.piece[row_n][element_n].y = start_y + self._y

    def draw(self):
        '''
        Draws all the blocks inside the shape.
        '''
        for row in self.piece:
            for element in row:
                if element is not None:
                    screen.draw.filled_rect(element, self.color)

    def pos_rotate(self):
        '''
        Rotate shape clockwise
        '''
        # Transpose the list
        transposed = [new_row for new_row in zip(*self.piece)]

        # Reverse elements in each row.
        rotated_shape = []
        for row in transposed:
            row = list(row)
            reversed_row = [element for element in reversed(row)]
            rotated_shape.append(reversed_row)

        self.piece = rotated_shape
        self.shape_grid_update()

    def neg_rotate(self):
        '''
        Rotate shape clockwise
        '''
        # Transpose the list
        transposed = [new_row for new_row in zip(*self.piece)]

        # Reverse elements in each row.
        rotated_shape = []
        for row in reversed(transposed):
            row = list(row)
            rotated_shape.append(row)

        self.piece = rotated_shape
        self.shape_grid_update()

    def move_down(self, speed):
        '''
        Moves the shape downwards in the speed passed in arg speed.
        speed is the number of pixels to move per update.
        '''
        self._y += speed
        self.shape_grid_update()

    def move_up(self, speed):
        '''
        Moves the shape upwards in the speed passed in arg speed.
        speed is the number of pixels to move per update.
        '''
        self._y -= speed
        self.shape_grid_update()

    def move_right(self):
        '''
        Moves the shape 1 step to the right.
        '''
        self._x += self.block_size
        self.shape_grid_update()

    def move_left(self):
        '''
        Moves the shape 1 step to the left
        '''
        self._x -= self.block_size
        self.shape_grid_update()

    def bottom(self):
        '''
        Gives the bottom pos of the first block in the first row.
        '''
        return self._y

    def left(self):
        '''
        Gives the left pos of the first block in the first row.
        '''
        x_pos = []
        for row in self.piece:
            for element in row:
                if element is not None:
                    x_pos.append(element.x)

        return min(x_pos)

    def right(self):
        '''
        Gives the right pos of the last block in the first row.
        '''
        x_pos = []
        for row in self.piece:
            for element in row:
                if element is not None:
                    x_pos.append(element.x)

        return max(x_pos)

    def top(self):
        '''
        Gives the top pos of the first block in the second row.
        '''
        # second row, first block will work fine.
        return self._y + (self.block_size * 2)

    def gameboard_collision(self, gameboard):
        '''
        Check if this piece have collided with any other piece on the bottom.
        '''
        for row in self.piece:
            for element in row:
                if element is not None:
                    if not gameboard.contains(element):
                        if element.bottom > gameboard.bottom:
                            return True
                        elif self.right() >= gameboard.right:
                            return True
                        elif self.left() < gameboard.left:
                            return True
        return False

    def shape_collision(self, other_pieces):
        '''
        Check if this piece have collided with any other piece on the bottom.
        '''

        # Unexpected behaviour. As it seems the bottom check is unnecessary
        for row in self.piece:
            for element in row:
                if element is not None:
                    collisions = element.collidelistall(other_pieces)
                    if collisions:
                        return True
        return False

    def get_rect(self):
        '''
        Returns a list of rects
        '''
        rect_list = []
        for row in self.piece:
            for element in row:
                if element is not None:
                    rect_list.append(element)
                    # rect_list.extend(row_rects)

        return rect_list

    def remove(self, block_list):
        '''
        Compare list of bricks against piece. If any match remove the entity.
        '''
        for block in block_list:
            for row in self.piece:
                try:
                    row.remove(block)
                except ValueError:
                    # Expected for some blocks.
                    # Not all blocks resides inside all bricks.
                    pass

    def __str__(self):
        return str(self.piece)


class OShape(Shape):
    '''
    Shape:
      xx
      xx
    '''

    def __init__(self, x_pos, size, color):
        Shape.__init__(self, x_pos, size, color)
        self.piece = [[None, None, None, None, None, None],
                      [None, None, None, None, None, None],
                      [None, None, Rect((0, 0), (size, size)),
                       Rect((0, 0), (size, size)), None, None],
                      [None, None, Rect((0, 0), (size, size)),
                       Rect((0, 0), (size, size)), None, None],
                      [None, None, None, None, None, None],
                      [None, None, None, None, None, None]]

        # Upate shape with set _x and _y values
        self.shape_grid_update()

    def pos_rotate(self):
        '''
        Try to rotate this one to victory is futile
        '''
        # This will override the method defined in the base class Shape
        pass

    def neg_rotate(self):
        '''
        Try to rotate this one to victory is futile
        '''
        # This will override the method defined in the base class Shape
        pass


class LShape(Shape):
    '''
    Shape:
      x
      x
      xx
    '''

    def __init__(self, x_pos, size, color):
        Shape.__init__(self, x_pos, size, color)
        self.piece = [[None, None, None, None, None, None],
                      [None, None, None, None, None, None],
                      [None, None, Rect((0, 0), (size, size)), None, None, None],
                      [None, None, Rect((0, 0), (size, size)), None, None, None],
                      [None, None, Rect((0, 0), (size, size)), Rect((0, 0), (size, size)), None, None],
                      [None, None, None, None, None, None]]

        # Upate shape with set _x and _y values
        self.shape_grid_update()


class IShape(Shape):
    '''
    Shape:
      x
      x
      x
      x
    '''
    def __init__(self, x_pos, size, color):
        Shape.__init__(self, x_pos, size, color)
        self.piece = [[None, None, None, None, None, None],
                      [None, None, Rect((0, 0), (size, size)), None, None, None],
                      [None, None, Rect((0, 0), (size, size)), None, None, None],
                      [None, None, Rect((0, 0), (size, size)), None, None, None],
                      [None, None, Rect((0, 0), (size, size)), None, None, None],
                      [None, None, None, None, None, None]]

        # Upate shape with set _x and _y values
        self.shape_grid_update()


class ZShape(Shape):
    '''
    Shape:
     xx
      xx
    '''
    def __init__(self, x_pos, size, color):
        Shape.__init__(self, x_pos, size, color)
        self.piece = [[None, None, None, None, None, None],
                      [None, None, None, None, None, None],
                      [None, None, Rect((0, 0), (size, size)), Rect((0, 0), (size, size)), None, None],
                      [None, None, None, Rect((0, 0), (size, size)), Rect((0, 0), (size, size)), None],
                      [None, None, None, None, None, None],
                      [None, None, None, None, None, None]]

        # Upate shape with set _x and _y values
        self.shape_grid_update()


class SShape(Shape):
    '''
    Shape:
      xx
     xx
    '''
    def __init__(self, x_pos, size, color):
        Shape.__init__(self, x_pos, size, color)
        self.piece = [[None, None, None, None, None, None],
                      [None, None, None, None, None, None],
                      [None, None, None, Rect((0, 0), (size, size)), Rect((0, 0), (size, size)), None],
                      [None, None, Rect((0, 0), (size, size)), Rect((0, 0), (size, size)), None, None],
                      [None, None, None, None, None, None],
                      [None, None, None, None, None, None]]

        # Upate shape with set _x and _y values
        self.shape_grid_update()


class TShape(Shape):
    '''
    Shape:
     xxx
      x
    '''
    def __init__(self, x_pos, size, color):
        Shape.__init__(self, x_pos, size, color)
        self.piece = [[None, None, None, None, None, None],
                      [None, None, None, None, None, None],
                      [None, None, Rect((0, 0), (size, size)),
                       Rect((0, 0), (size, size)),
                       Rect((0, 0), (size, size)), None],
                      [None, None, None, Rect((0, 0), (size, size)), None, None],
                      [None, None, None, None, None, None],
                      [None, None, None, None, None, None]]

        # Upate shape with set _x and _y values
        self.shape_grid_update()


class JShape(Shape):
    '''
    Shape:
      x
      x
     xx
    '''
    def __init__(self, x_pos, size, color):
        Shape.__init__(self, x_pos, size, color)
        self.piece = [[None, None, None, None, None, None],
                      [None, None, None, None, None, None],
                      [None, None, Rect((0, 0), (size, size)), None, None, None],
                      [None, None, Rect((0, 0), (size, size)), None, None, None],
                      [None, Rect((0, 0), (size, size)), Rect((0, 0), (size, size)),
                       None, None, None],
                      [None, None, None, None, None, None]]

        # Upate shape with set _x and _y values
        self.shape_grid_update()


class Bricks(object):
    '''
    There is the following kind of bricks
    1: O-shaped
    2: L-shaped
    3: I-shaped
    4: Z-shaped
    5: S-shaped
    6: T-shaped
    7: J-shaped

    There is the following list of colors.
    1: Red (0xFF, 0x0, 0x0)
    2: Yellow (0xF9, 0xFF, 0x0)
    3: Magenta (0xFF, 0x0, 0xFF)
    4: Blue (0x42, 0x0, 0xFF)
    5: Cyan (0x0, 0xFF, 0xFF)
    6: Lime (0x1A, 0xFF, 0x0)
    7: Orange (0xFF, 0xA1, 0x0)

    The Brick class will function as a bridge for all the shapes
    and as a container for bricks.
    '''
    def __init__(self, block_size):
        '''
        Valid data for parameter shape is:
        1, 2, 3, 4
        '''
        self.block_size = block_size
        self.bricklist = []

        # A dict with references to the constructor for each available shape.
        self.shape_list = {1: OShape, 2: LShape, 3: IShape, 4: ZShape,
                           5: SShape, 6: TShape, 7: JShape}
        # A-Arcade color theme
        self.color_palette = {1: (0xFF, 0x0, 0x0), 2: (0xF9, 0xFF, 0x0),
                              3: (0xFF, 0x0, 0xFF), 4: (0x42, 0x0, 0xFF),
                              5: (0x0, 0xFF, 0xFF), 6: (0x1A, 0xFF, 0x0),
                              7: (0xFF, 0xA1, 0x0)}

    def new_brick(self, shape, color, pos, size):
        '''
        Adds a new brick to the list
        '''
        # Check if parameter shape is valid else create the Brick piece.
        if shape not in self.shape_list:
            raise ValueError('Not in shape_list: {0}'.format(shape))
        elif color not in self.color_palette:
            raise ValueError('Not in color_palette: {0}'.format(color))
        else:
            self.bricklist.append(self.shape_list[shape](pos, size, self.color_palette[color]))

    @property
    def bottom(self):
        '''
        Returns the position of the bottom.
        '''
        return self.bricklist[-1].bottom()

    def move_list_down(self, move_list):
        '''
        Moves the list of blocks one step down.
        '''
        for block in move_list:
            block.y += self.block_size

    def move_active_down(self, speed):
        '''
        Move active brick down in a given speed
        '''
        self.bricklist[-1].move_down(speed)

    def move_active_up(self, speed):
        '''
        Move active brick up with a given speed
        '''
        self.bricklist[-1].move_up(speed)

    def move_active_right(self):
        '''
        Move brick right one step
        '''
        self.bricklist[-1].move_right()

    def move_active_left(self):
        '''
        Move active brick left one step
        '''
        self.bricklist[-1].move_left()

    def pos_rotate_active(self):
        '''
        Rotates active brick 90deg.
        '''
        self.bricklist[-1].pos_rotate()

    def neg_rotate_active(self):
        '''
        Rotates active brick -90deg.
        '''
        self.bricklist[-1].neg_rotate()

    def draw(self):
        '''
        Draw shape on to the screen
        '''
        for brick in self.bricklist:
            brick.draw()

    def gameboard_collision(self, gameboard):
        '''
        Check if Brick have hit gameboard bottom
        '''
        return self.bricklist[-1].gameboard_collision(gameboard)

    def shape_collision(self):
        '''
        other_pieces is expected to be a list of type Rect
        If piece have collided with other piece at bottom.
        returns true.
        '''
        for brick in self.bricklist[:-1]:
            if self.bricklist[-1].shape_collision(brick.get_rect()):
                return True

        return False

    def bounds_out(self, gameboard):
        '''
        Returns True if any of the active brick is out side of the
        gameboard when called.
        Else False
        '''
        block_list = self.bricklist[-1].get_rect()
        game_rect = gameboard.get_rect()

        # Check if all the blocks from the brick is inside the gameboard.
        # If not, then return True. otherwise the function will return False
        for block in block_list:
            if not game_rect.contains(block):
                return True

        return False

    def remove(self, block_list):
        '''
        If brick contains any of the bricks in list. It will remove them
        '''
        for brick in self.bricklist:
            brick.remove(block_list)

    def get_active_rect(self):
        '''
        Returns rectangle objects from brick.
        '''
        return self.bricklist[-1].get_rect()

    def get_rect_list(self):
        '''
        Helper method to compile a list of rects from a list of bricks
        '''
        rect_list = []
        for brick in self.bricklist:
            rect_list.extend(brick.get_rect())
        return rect_list


class ScoreBoard(object):
    '''
    Shows the game the current score
    '''
    def __init__(self, pos):
        self.title = 'Score'
        self.pos_x, self.pos_y = pos
        self.current_score = 0

    def add_to_score(self, points):
        '''
        Takes points as argument and adds it to the current score
        '''
        # Calculate score multiplier
        multiplier = points / 10

        # Set new score
        self.current_score += int(points * multiplier)

    def draw(self):
        '''
        Draws title and current score to surface
        '''
        screen.draw.text(self.title,
                         (self.pos_x, self.pos_y),
                         color=(0xDE, 0xAD, 0xFF),
                         fontsize=20)
        screen.draw.text(str(self.current_score),
                         (self.pos_x, self.pos_y + 20),
                         color=(0x00, 0xAD, 0xFF),
                         fontsize=20)


class GameBoard(object):
    '''
    Implements the play area and is judge that decides what rectangles
    that should be removed and which dead bricks that should move down.
    '''
    def __init__(self, position, gameboard_size, block_size, color):
        pos_x, pos_y = position
        self._pos_x = pos_x
        self._pos_y = pos_y
        self.cols, self.rows = gameboard_size
        self.block_size = block_size
        width = self.cols * self.block_size
        height = self.rows * self.block_size
        self._board = Rect(position, (width, height))
        self.color = color

    def center(self):
        '''
        Returns the centrum pos in the X axis.
        '''
        center = (self._board.width / 2) + self._pos_x
        return center

    def get_rect(self):
        '''
        Returns a Rect object of the GameBoard
        '''
        return self._board

    def draw(self):
        '''
        To be called when we want it to be drawn onto the screen surface.
        '''
        screen.draw.filled_rect(self._board, self.color)

    def ruler(self, rects):
        '''
        Takes a list of bricks. Returns a list of rects that should be removed
        from the first row that it encounters
        and a list of rects that should be moved down a row.

        If nothing to report, it returns a tuple of (None, None)
        '''
        # Get a list of every block that is available on the gameboard.
        blocks = rects

        # A complete row, is a row with the same amount of blocks
        # as there is columns
        complete_row = self.cols

        # Create a ruler that can be used to check if a row is full of blocks.
        ruler_rect = Rect((self._pos_x, self._board.bottom - self.block_size),
                          (self._board.width, self.block_size))

        # If a row is full, we need to figure out if there is
        # any blocks above that row that should be moved down.
        other_rect = Rect((self._pos_x, self._pos_y),
                          (self._board.width, self._board.height))
        remove_list = []
        move_down_list = []

        # Uses the ruler rect to check if all the columns in row is filled.
        # If so, it creates a list of blocks that should be removed
        # and a list of blocks that should be moved one row down.
        for _ in range(0, self.rows):
            # Area to capture rects above the row for which the ruler is at.
            other_rect.height -= self.block_size

            collision_indices = ruler_rect.collidelistall(blocks)
            move_down_indices = other_rect.collidelistall(blocks)
            if len(collision_indices) == complete_row:
                for index in collision_indices:
                    remove_list.append(blocks[index])
                for index in move_down_indices:
                    move_down_list.append(blocks[index])

                return (remove_list, move_down_list)

            # Step the ruler to next row.
            ruler_rect.y -= self.block_size

        #No full row, no blocks the be moved down.
        return (None, None)


class Game(object):
    '''
    Game board, bricks and controls.
    '''
    def __init__(self, x, y, cols, rows):
        '''
        '''
        # Block setting
        self.block_size = 20

        self._xpos = x
        self._ypos = y
        self._cols = cols
        self._rows = rows

        self.width = cols * self.block_size
        self.height = rows * self.block_size
        self.gameboard = GameBoard((x, y), (cols, rows), self.block_size, (0x00, 0x00, 0x00))

        # Scoreboard
        margin = 10
        self.scoreboard = ScoreBoard((self._xpos + self.width + margin,
                                      self._ypos))

        # Contains a list of previously active bricks.
        self.bricks = Bricks(self.block_size)
        # Speed affects all moving objects, such as the active_brick
        self.speed = 1

        # A lock that will be False once the game hits the state of game over.
        self.game_active = True
        self.game_pause = False

        self.new_brick()

    def new_brick(self):
        '''
        Creates a new brick when previous get locked in place
        '''
        self.bricks.new_brick(random.randint(1, 7),
                              random.randint(1, 7),
                              self.gameboard.center(),
                              self.block_size)

    def draw(self):
        '''
        Draw game
        '''
        self.gameboard.draw()
        self.bricks.draw()
        self.scoreboard.draw()

        if not self.game_active:
            screen.draw.text('Game Over', (40, 100), color=(0xDE, 0xAD, 0xFF), fontsize=80)

    def remove_complete_rows(self):
        '''
        For every complete row, request to remove it.
        The request is sent to each brick in the game.
        '''
        points = 0
        # Check if row/rows became complete
        while True:
            remove_list, move_list = self.gameboard.ruler(self.bricks.get_rect_list())
            if remove_list:
                self.bricks.remove(remove_list)
                points += len(remove_list)

            if move_list:
                self.bricks.move_list_down(move_list)

            else:
                break

        # Add points to scoreboard
        self.scoreboard.add_to_score(points)

    def toggle_pause(self):
        '''
        Pause/Un-pause the game
        '''
        self.game_pause = not self.game_pause

    def proceed(self):
        '''
        Is called to change the state of the game.
        '''
        # Is brick active, if not, kill it and create a new one.
        if self.found_collision():
            self.bricks.move_active_up(self.speed)
            self.remove_complete_rows()
            # Game over if brick is outside
            if self.bricks.bounds_out(self.gameboard):
                # Draw game over to the screen and raise GameOver
                self.game_active = False
                raise GameOver
            else:
                self.new_brick()

        else:
            self.bricks.move_active_down(self.speed)

    def found_collision(self):
        '''
        Check if active brick have collided with anything.
        '''
        if self.bricks.gameboard_collision(self.gameboard.get_rect()):
            return True
        elif self.bricks.shape_collision():
            return True
        else:
            return False

    def controls(self, direction):
        '''
        Will be called for a given intervall. This prevents the blocks from
        flying from left to right in high speed
        '''

        # Check if user want to move brick side ways.
        if direction == 'right':
            self.bricks.move_active_right()
        elif direction == 'left':
            self.bricks.move_active_left()
        elif direction == 'down':
            self.bricks.move_active_down(self.speed * 10)
        elif direction == 'pos_rotate':
            self.bricks.pos_rotate_active()
        elif direction == 'neg_rotate':
            self.bricks.neg_rotate_active()

        # Damage control after collision
        if self.found_collision():
            if direction == 'right':
                self.bricks.move_active_left()
            elif direction == 'left':
                self.bricks.move_active_right()
            elif direction == 'down':
                self.bricks.move_active_up(self.speed * 10)
            elif direction == 'pos_rotate':
                self.bricks.neg_rotate_active()
            elif direction == 'neg_rotate':
                self.bricks.pos_rotate_active()

    def reset(self):
        '''
        Restarts the game
        '''
        Game.__init__(self, self._xpos, self._ypos, self._cols, self._rows)


WIDTH = 500
HEIGHT = 800

music.set_volume(0.5)
music.play('hhavok-main')

# Create a gamecontroller and a gameboard with a width and a height
# Width and height is given in columns and rows.
TETRA_PUZZLE = Game(100, 50, 10, 20)

# Global that keeps track on wheen to increase the speed on
# right and left movement
SPEEDOMETER_LEFT = 0
SPEEDOMETER_RIGHT = 0

def control_tick():
    '''
    Gets triggerad by clock.
    Tells the game betris what the user want to do, if any.
    '''
    global SPEEDOMETER_RIGHT
    global SPEEDOMETER_LEFT

    if TETRA_PUZZLE.game_active:
        if keyboard.space:
            TETRA_PUZZLE.controls('down')
        elif keyboard.right:
            if SPEEDOMETER_LEFT:
                SPEEDOMETER_LEFT = 0
            SPEEDOMETER_RIGHT += 0.01
            TETRA_PUZZLE.controls('right')
        elif keyboard.left:
            if SPEEDOMETER_RIGHT:
                SPEEDOMETER_RIGHT = 0
            SPEEDOMETER_LEFT += 0.01
            TETRA_PUZZLE.controls('left')

def on_key_down(key):
    # if key is keys.RIGHT:
        # TETRA_PUZZLE.controls('right')
    # elif key is keys.LEFT:
        # TETRA_PUZZLE.controls('left')
    if key is keys.R:
        TETRA_PUZZLE.reset()
    elif key is keys.P:
        TETRA_PUZZLE.toggle_pause()
    elif key is keys.UP:
        TETRA_PUZZLE.controls('pos_rotate')
    elif key is keys.DOWN:
        TETRA_PUZZLE.controls('neg_rotate')


# Setup time between each check of user controls.
clock.schedule_interval(control_tick, (0.05 - SPEEDOMETER_RIGHT) - SPEEDOMETER_LEFT)

def update():
    '''
    Update block position.
    '''
    if TETRA_PUZZLE.game_active and not TETRA_PUZZLE.game_pause:
        try:
            TETRA_PUZZLE.proceed()
        except GameOver:
            draw()

def draw():
    '''
    Draw game
    '''
    screen.fill((0xFF, 0xFF, 0xFF))
    TETRA_PUZZLE.draw()



import unittest
import pygame
from pgzero.rect import ZRect as Rect


class RectTypeTest( unittest.TestCase ):
    def testConstructionXYWidthHeight( self ):
        r = Rect(1,2,3,4)
        self.assertEqual( 1, r.left )
        self.assertEqual( 2, r.top )
        self.assertEqual( 3, r.width )
        self.assertEqual( 4, r.height )

    def testConstructionTopLeftSize( self ):
        r = Rect( (1,2), (3,4) )
        self.assertEqual( 1, r.left )
        self.assertEqual( 2, r.top )
        self.assertEqual( 3, r.width )
        self.assertEqual( 4, r.height )

    def testCalculatedAttributes( self ):
        r = Rect( 1, 2, 3, 4 )

        self.assertEqual( r.left+r.width, r.right )
        self.assertEqual( r.top+r.height, r.bottom )
        self.assertEqual( (r.width,r.height), r.size )
        self.assertEqual( (r.left,r.top), r.topleft )
        self.assertEqual( (r.right,r.top), r.topright )
        self.assertEqual( (r.left,r.bottom), r.bottomleft )
        self.assertEqual( (r.right,r.bottom), r.bottomright )

        midx = r.left + r.width / 2
        midy = r.top + r.height / 2

        self.assertEqual( midx, r.centerx )
        self.assertEqual( midy, r.centery )
        self.assertEqual( (r.centerx,r.centery), r.center )
        self.assertEqual( (r.centerx,r.top), r.midtop )
        self.assertEqual( (r.centerx,r.bottom), r.midbottom )
        self.assertEqual( (r.left,r.centery), r.midleft )
        self.assertEqual( (r.right,r.centery), r.midright )

    def test_normalize( self ):
        r = Rect( 1, 2, -3, -6 )
        r2 = Rect(r)
        r2.normalize()
        self.assertTrue( r2.width >= 0 )
        self.assertTrue( r2.height >= 0 )
        self.assertEqual( (abs(r.width),abs(r.height)), r2.size )
        self.assertEqual( (-2,-4), r2.topleft )

    def test_left( self ):
        """Changing the left attribute moves the rect and does not change
           the rect's width
        """
        r = Rect( 1, 2, 3, 4 )
        new_left = 10

        r.left = new_left
        self.assertEqual( new_left, r.left )
        self.assertEqual( Rect(new_left,2,3,4), r )

    def test_right( self ):
        """Changing the right attribute moves the rect and does not change
           the rect's width
        """
        r = Rect( 1, 2, 3, 4 )
        new_right = r.right + 20
        expected_left = r.left + 20
        old_width = r.width

        r.right = new_right
        self.assertEqual( new_right, r.right )
        self.assertEqual( expected_left, r.left )
        self.assertEqual( old_width, r.width )

    def test_top( self ):
        """Changing the top attribute moves the rect and does not change
           the rect's width
        """
        r = Rect( 1, 2, 3, 4 )
        new_top = 10

        r.top = new_top
        self.assertEqual( Rect(1,new_top,3,4), r )
        self.assertEqual( new_top, r.top )

    def test_bottom( self ):
        """Changing the bottom attribute moves the rect and does not change
           the rect's height
        """
        r = Rect( 1, 2, 3, 4 )
        new_bottom = r.bottom + 20
        expected_top = r.top + 20
        old_height = r.height

        r.bottom = new_bottom
        self.assertEqual( new_bottom, r.bottom )
        self.assertEqual( expected_top, r.top )
        self.assertEqual( old_height, r.height )

    def test_centerx( self ):
        """Changing the centerx attribute moves the rect and does not change
           the rect's width
        """
        r = Rect( 1, 2, 3, 4 )
        new_centerx = r.centerx + 20
        expected_left = r.left + 20
        old_width = r.width

        r.centerx = new_centerx
        self.assertEqual( new_centerx, r.centerx )
        self.assertEqual( expected_left, r.left )
        self.assertEqual( old_width, r.width )

    def test_centery( self ):
        """Changing the centerx attribute moves the rect and does not change
           the rect's width
        """
        r = Rect( 1, 2, 3, 4 )
        new_centery = r.centery + 20
        expected_top = r.top + 20
        old_height = r.height

        r.centery = new_centery
        self.assertEqual( new_centery, r.centery )
        self.assertEqual( expected_top, r.top )
        self.assertEqual( old_height, r.height )

    def test_topleft( self ):
        """Changing the topleft attribute moves the rect and does not change
           the rect's size
        """
        r = Rect( 1, 2, 3, 4 )
        new_topleft = (r.left+20,r.top+30)
        old_size = r.size

        r.topleft = new_topleft
        self.assertEqual( new_topleft, r.topleft )
        self.assertEqual( old_size, r.size )

    def test_bottomleft( self ):
        """Changing the bottomleft attribute moves the rect and does not change
           the rect's size
        """
        r = Rect( 1, 2, 3, 4 )
        new_bottomleft = (r.left+20,r.bottom+30)
        expected_topleft = (r.left+20,r.top+30)
        old_size = r.size

        r.bottomleft = new_bottomleft
        self.assertEqual( new_bottomleft, r.bottomleft )
        self.assertEqual( expected_topleft, r.topleft )
        self.assertEqual( old_size, r.size )

    def test_topright( self ):
        """Changing the bottomleft attribute moves the rect and does not change
           the rect's size
        """
        r = Rect( 1, 2, 3, 4 )
        new_topright = (r.right+20,r.top+30)
        expected_topleft = (r.left+20,r.top+30)
        old_size = r.size

        r.topright = new_topright
        self.assertEqual( new_topright, r.topright )
        self.assertEqual( expected_topleft, r.topleft )
        self.assertEqual( old_size, r.size )

    def test_bottomright( self ):
        """Changing the bottomright attribute moves the rect and does not change
           the rect's size
        """
        r = Rect( 1, 2, 3, 4 )
        new_bottomright = (r.right+20,r.bottom+30)
        expected_topleft = (r.left+20,r.top+30)
        old_size = r.size

        r.bottomright = new_bottomright
        self.assertEqual( new_bottomright, r.bottomright )
        self.assertEqual( expected_topleft, r.topleft )
        self.assertEqual( old_size, r.size )

    def test_center( self ):
        """Changing the center attribute moves the rect and does not change
           the rect's size
        """
        r = Rect( 1, 2, 3, 4 )
        new_center = (r.centerx+20,r.centery+30)
        expected_topleft = (r.left+20,r.top+30)
        old_size = r.size

        r.center = new_center
        self.assertEqual( new_center, r.center )
        self.assertEqual( expected_topleft, r.topleft )
        self.assertEqual( old_size, r.size )

    def test_midleft( self ):
        """Changing the midleft attribute moves the rect and does not change
           the rect's size
        """
        r = Rect( 1, 2, 3, 4 )
        new_midleft = (r.left+20,r.centery+30)
        expected_topleft = (r.left+20,r.top+30)
        old_size = r.size

        r.midleft = new_midleft
        self.assertEqual( new_midleft, r.midleft )
        self.assertEqual( expected_topleft, r.topleft )
        self.assertEqual( old_size, r.size )

    def test_midright( self ):
        """Changing the midright attribute moves the rect and does not change
           the rect's size
        """
        r = Rect( 1, 2, 3, 4 )
        new_midright= (r.right+20,r.centery+30)
        expected_topleft = (r.left+20,r.top+30)
        old_size = r.size

        r.midright = new_midright
        self.assertEqual( new_midright, r.midright )
        self.assertEqual( expected_topleft, r.topleft )
        self.assertEqual( old_size, r.size )

    def test_midtop( self ):
        """Changing the midtop attribute moves the rect and does not change
           the rect's size
        """
        r = Rect( 1, 2, 3, 4 )
        new_midtop= (r.centerx+20,r.top+30)
        expected_topleft = (r.left+20,r.top+30)
        old_size = r.size

        r.midtop = new_midtop
        self.assertEqual( new_midtop, r.midtop )
        self.assertEqual( expected_topleft, r.topleft )
        self.assertEqual( old_size, r.size )

    def test_midbottom( self ):
        """Changing the midbottom attribute moves the rect and does not change
           the rect's size
        """
        r = Rect( 1, 2, 3, 4 )
        new_midbottom = (r.centerx+20,r.bottom+30)
        expected_topleft = (r.left+20,r.top+30)
        old_size = r.size

        r.midbottom = new_midbottom
        self.assertEqual( new_midbottom, r.midbottom )
        self.assertEqual( expected_topleft, r.topleft )
        self.assertEqual( old_size, r.size )

    def test_width( self ):
        "Changing the width resizes the rect from the top-left corner"
        r = Rect( 1, 2, 3, 4 )
        new_width = 10
        old_topleft = r.topleft
        old_height = r.height

        r.width = new_width
        self.assertEqual( new_width, r.width )
        self.assertEqual( old_height, r.height )
        self.assertEqual( old_topleft, r.topleft )

    def test_height( self ):
        "Changing the height resizes the rect from the top-left corner"
        r = Rect( 1, 2, 3, 4 )
        new_height = 10
        old_topleft = r.topleft
        old_width = r.width

        r.height = new_height
        self.assertEqual( new_height, r.height )
        self.assertEqual( old_width, r.width )
        self.assertEqual( old_topleft, r.topleft )

    def test_size( self ):
        "Changing the size resizes the rect from the top-left corner"
        r = Rect( 1, 2, 3, 4 )
        new_size = (10,20)
        old_topleft = r.topleft

        r.size = new_size
        self.assertEqual( new_size, r.size )
        self.assertEqual( old_topleft, r.topleft )

    def test_contains( self ):
        r = Rect( 1, 2, 3, 4 )

        self.assertTrue( r.contains( Rect( 2, 3, 1, 1 ) ),
                         "r does not contain Rect(2,3,1,1)" )
        self.assertTrue( r.contains( Rect(r) ),
                         "r does not contain the same rect as itself" )
        self.assertTrue( r.contains( Rect(2,3,0,0) ),
                         "r does not contain an empty rect within its bounds" )
        self.assertFalse( r.contains( Rect(0,0,1,2) ),
                     "r contains Rect(0,0,1,2)" )
        self.assertFalse( r.contains( Rect(4,6,1,1) ),
                     "r contains Rect(4,6,1,1)" )
        self.assertFalse( r.contains( Rect(4,6,0,0) ),
                     "r contains Rect(4,6,0,0)" )

    def test_collidepoint( self ):
        r = Rect( 1, 2, 3, 4 )

        self.assertTrue( r.collidepoint( r.left, r.top ),
                         "r does not collide with point (left,top)" )
        self.assertFalse( r.collidepoint( r.left-1, r.top ),
                     "r collides with point (left-1,top)"  )
        self.assertFalse( r.collidepoint( r.left, r.top-1 ),
                     "r collides with point (left,top-1)"  )
        self.assertFalse( r.collidepoint( r.left-1,r.top-1 ),
                     "r collides with point (left-1,top-1)"  )

        self.assertTrue( r.collidepoint( r.right-1, r.bottom-1 ),
                         "r does not collide with point (right-1,bottom-1)")
        self.assertFalse( r.collidepoint( r.right, r.bottom ),
                     "r collides with point (right,bottom)" )
        self.assertFalse( r.collidepoint( r.right-1, r.bottom ),
                     "r collides with point (right-1,bottom)" )
        self.assertFalse( r.collidepoint( r.right, r.bottom-1 ),
                     "r collides with point (right,bottom-1)" )

    def test_inflate__larger( self ):
        "The inflate method inflates around the center of the rectangle"
        r = Rect( 2, 4, 6, 8 )
        r2 = r.inflate( 4, 6 )

        self.assertEqual( r.center, r2.center )
        self.assertEqual( r.left-2, r2.left )
        self.assertEqual( r.top-3, r2.top )
        self.assertEqual( r.right+2, r2.right )
        self.assertEqual( r.bottom+3, r2.bottom )
        self.assertEqual( r.width+4, r2.width )
        self.assertEqual( r.height+6, r2.height )

    def test_inflate__smaller( self ):
        "The inflate method inflates around the center of the rectangle"
        r = Rect( 2, 4, 6, 8 )
        r2 = r.inflate( -4, -6 )

        self.assertEqual( r.center, r2.center )
        self.assertEqual( r.left+2, r2.left )
        self.assertEqual( r.top+3, r2.top )
        self.assertEqual( r.right-2, r2.right )
        self.assertEqual( r.bottom-3, r2.bottom )
        self.assertEqual( r.width-4, r2.width )
        self.assertEqual( r.height-6, r2.height )

    def test_inflate_ip__larger( self ):
        "The inflate_ip method inflates around the center of the rectangle"
        r = Rect( 2, 4, 6, 8 )
        r2 = Rect( r )
        r2.inflate_ip( -4, -6 )

        self.assertEqual( r.center, r2.center )
        self.assertEqual( r.left+2, r2.left )
        self.assertEqual( r.top+3, r2.top )
        self.assertEqual( r.right-2, r2.right )
        self.assertEqual( r.bottom-3, r2.bottom )
        self.assertEqual( r.width-4, r2.width )
        self.assertEqual( r.height-6, r2.height )

    def test_inflate_ip__smaller( self ):
        "The inflate method inflates around the center of the rectangle"
        r = Rect( 2, 4, 6, 8 )
        r2 = Rect( r )
        r2.inflate_ip( -4, -6 )

        self.assertEqual( r.center, r2.center )
        self.assertEqual( r.left+2, r2.left )
        self.assertEqual( r.top+3, r2.top )
        self.assertEqual( r.right-2, r2.right )
        self.assertEqual( r.bottom-3, r2.bottom )
        self.assertEqual( r.width-4, r2.width )
        self.assertEqual( r.height-6, r2.height )

    def test_clamp( self ):
        r = Rect(10, 10, 10, 10)
        c = Rect(19, 12, 5, 5).clamp(r)
        self.assertEqual(c.right, r.right)
        self.assertEqual(c.top, 12)
        c = Rect(1, 2, 3, 4).clamp(r)
        self.assertEqual(c.topleft, r.topleft)
        c = Rect(5, 500, 22, 33).clamp(r)
        self.assertEqual(c.center, r.center)

    def test_clamp_ip( self ):
        r = Rect(10, 10, 10, 10)
        c = Rect(19, 12, 5, 5)
        c.clamp_ip(r)
        self.assertEqual(c.right, r.right)
        self.assertEqual(c.top, 12)
        c = Rect(1, 2, 3, 4)
        c.clamp_ip(r)
        self.assertEqual(c.topleft, r.topleft)
        c = Rect(5, 500, 22, 33)
        c.clamp_ip(r)
        self.assertEqual(c.center, r.center)

    def test_clip( self ):
        r1 = Rect( 1, 2, 3, 4 )
        self.assertEqual( Rect( 1, 2, 2, 2 ), r1.clip( Rect(0,0,3,4) ) )
        self.assertEqual( Rect( 2, 2, 2, 4 ), r1.clip( Rect(2,2,10,20) ) )
        self.assertEqual( Rect(2,3,1,2), r1.clip( Rect(2,3,1,2) ) )
        self.assertEqual( (0,0), r1.clip(20,30,5,6).size )
        self.assertEqual( r1, r1.clip( Rect(r1) ),
                          "r1 does not clip an identical rect to itself" )

    def test_move( self ):
        r = Rect( 1, 2, 3, 4 )
        move_x = 10
        move_y = 20
        r2 = r.move( move_x, move_y )
        expected_r2 = Rect(r.left+move_x,r.top+move_y,r.width,r.height)
        self.assertEqual( expected_r2, r2 )

    def test_move_ip( self ):
        r = Rect( 1, 2, 3, 4 )
        r2 = Rect( r )
        move_x = 10
        move_y = 20
        r2.move_ip( move_x, move_y )
        expected_r2 = Rect(r.left+move_x,r.top+move_y,r.width,r.height)
        self.assertEqual( expected_r2, r2 )

    def test_union( self ):
        r1 = Rect( 1, 1, 1, 2 )
        r2 = Rect( -2, -2, 1, 2 )
        self.assertEqual( Rect( -2, -2, 4, 5 ), r1.union(r2) )

    def test_union__with_identical_Rect( self ):
        r1 = Rect( 1, 2, 3, 4 )
        self.assertEqual( r1, r1.union( Rect(r1) ) )

    def test_union_ip( self ):
        r1 = Rect( 1, 1, 1, 2 )
        r2 = Rect( -2, -2, 1, 2 )
        r1.union_ip(r2)
        self.assertEqual( Rect( -2, -2, 4, 5 ), r1 )

    def test_unionall( self ):
        r1 = Rect( 0, 0, 1, 1 )
        r2 = Rect( -2, -2, 1, 1 )
        r3 = Rect( 2, 2, 1, 1 )

        r4 = r1.unionall( [r2,r3] )
        self.assertEqual( Rect(-2, -2, 5, 5), r4 )

    def test_unionall_ip( self ):
        r1 = Rect( 0, 0, 1, 1 )
        r2 = Rect( -2, -2, 1, 1 )
        r3 = Rect( 2, 2, 1, 1 )

        r1.unionall_ip( [r2,r3] )
        self.assertEqual( Rect(-2, -2, 5, 5), r1 )



    def test_colliderect( self ):
        r1 = Rect(1,2,3,4)
        self.assertTrue( r1.colliderect( Rect(0,0,2,3) ),
                         "r1 does not collide with Rect(0,0,2,3)" )
        self.assertFalse( r1.colliderect( Rect(0,0,1,2) ),
                     "r1 collides with Rect(0,0,1,2)" )
        self.assertFalse( r1.colliderect( Rect(r1.right,r1.bottom,2,2) ),
                     "r1 collides with Rect(r1.right,r1.bottom,2,2)" )
        self.assertTrue( r1.colliderect( Rect(r1.left+1,r1.top+1,
                                              r1.width-2,r1.height-2) ),
                         "r1 does not collide with Rect(r1.left+1,r1.top+1,"+
                         "r1.width-2,r1.height-2)" )
        self.assertTrue( r1.colliderect( Rect(r1.left-1,r1.top-1,
                                              r1.width+2,r1.height+2) ),
                         "r1 does not collide with Rect(r1.left-1,r1.top-1,"+
                         "r1.width+2,r1.height+2)" )
        self.assertTrue( r1.colliderect( Rect(r1) ),
                         "r1 does not collide with an identical rect" )
        self.assertFalse( r1.colliderect( Rect(r1.right,r1.bottom,0,0) ),
                     "r1 collides with Rect(r1.right,r1.bottom,0,0)" )
        self.assertFalse( r1.colliderect( Rect(r1.right,r1.bottom,1,1) ),
                     "r1 collides with Rect(r1.right,r1.bottom,1,1)" )

    def testEquals( self ):
        """ check to see how the rect uses __eq__
        """
        r1 = Rect(1,2,3,4)
        r2 = Rect(10,20,30,40)
        r3 = (10,20,30,40)
        r4 = Rect(10,20,30,40)

        class foo (Rect):
            def __eq__(self,other):
                return id(self) == id(other)
            def __ne__(self,other):
                return id(self) != id(other)

        class foo2 (Rect):
            pass

        r5 = foo(10,20,30,40)
        r6 = foo2(10,20,30,40)

        self.assertNotEqual(r5, r2)

        # because we define equality differently for this subclass.
        self.assertEqual(r6, r2)


        rect_list = [r1,r2,r3,r4,r6]

        # see if we can remove 4 of these.
        rect_list.remove(r2)
        rect_list.remove(r2)
        rect_list.remove(r2)
        rect_list.remove(r2)
        self.assertRaises(ValueError, rect_list.remove, r2)

    def test_collidedict(self):

        # __doc__ (as of 2008-08-02) for pygame.rect.Rect.collidedict:

          # Rect.collidedict(dict): return (key, value)
          # test if one rectangle in a dictionary intersects
          #
          # Returns the key and value of the first dictionary value that
          # collides with the Rect. If no collisions are found, None is
          # returned.
          #
          # Rect objects are not hashable and cannot be used as keys in a
          # dictionary, only as values.

        r = Rect(1, 1, 10, 10)
        r1 = Rect(1, 1, 10, 10)
        r2 = Rect(50, 50, 10, 10)
        r3 = Rect(70, 70, 10, 10)
        r4 = Rect(61, 61, 10, 10)

        d = {1: r1, 2: r2, 3: r3}

        rects_values = 1
        val = r.collidedict(d, rects_values)
        self.assertTrue(val)
        self.assertEqual(len(val), 2)
        self.assertEqual(val[0], 1)
        self.assertEqual(val[1], r1)

        none_d = {2: r2, 3: r3}
        none_val = r.collidedict(none_d, rects_values)
        self.assertFalse(none_val)

        barely_d = {1: r1, 2: r2, 3: r3}
        k3, v3 = r4.collidedict(barely_d, rects_values)
        self.assertEqual(k3, 3)
        self.assertEqual(v3, r3)


    def test_collidedictall(self):

        # __doc__ (as of 2008-08-02) for pygame.rect.Rect.collidedictall:

          # Rect.collidedictall(dict): return [(key, value), ...]
          # test if all rectangles in a dictionary intersect
          #
          # Returns a list of all the key and value pairs that intersect with
          # the Rect. If no collisions are found an empty dictionary is
          # returned.
          #
          # Rect objects are not hashable and cannot be used as keys in a
          # dictionary, only as values.

        r = Rect(1, 1, 10, 10)

        r2 = Rect(1, 1, 10, 10)
        r3 = Rect(5, 5, 10, 10)
        r4 = Rect(10, 10, 10, 10)
        r5 = Rect(50, 50, 10, 10)

        rects_values = 1
        d = {2: r2}
        l = r.collidedictall(d, rects_values)
        self.assertEqual(l, [(2, r2)])

        d2 = {2: r2, 3: r3, 4: r4, 5: r5}
        l2 = r.collidedictall(d2, rects_values)
        self.assertEqual(l2, [(2, r2), (3, r3), (4, r4)])

    def test_collidelist(self):

        # __doc__ (as of 2008-08-02) for pygame.rect.Rect.collidelist:

          # Rect.collidelist(list): return index
          # test if one rectangle in a list intersects
          #
          # Test whether the rectangle collides with any in a sequence of
          # rectangles. The index of the first collision found is returned. If
          # no collisions are found an index of -1 is returned.

        r = Rect(1, 1, 10, 10)
        l = [Rect(50, 50, 1, 1), Rect(5, 5, 10, 10), Rect(15, 15, 1, 1)]

        self.assertEqual(r.collidelist(l), 1)

        f = [Rect(50, 50, 1, 1), (100, 100, 4, 4)]
        self.assertEqual(r.collidelist(f), -1)


    def test_collidelistall(self):

        # __doc__ (as of 2008-08-02) for pygame.rect.Rect.collidelistall:

          # Rect.collidelistall(list): return indices
          # test if all rectangles in a list intersect
          #
          # Returns a list of all the indices that contain rectangles that
          # collide with the Rect. If no intersecting rectangles are found, an
          # empty list is returned.

        r = Rect(1, 1, 10, 10)

        l = [
            Rect(1, 1, 10, 10),
            Rect(5, 5, 10, 10),
            Rect(15, 15, 1, 1),
            Rect(2, 2, 1, 1),
        ]
        self.assertEqual(r.collidelistall(l), [0, 1, 3])

        f = [Rect(50, 50, 1, 1), Rect(20, 20, 5, 5)]
        self.assertFalse(r.collidelistall(f))


    def test_fit(self):

        # __doc__ (as of 2008-08-02) for pygame.rect.Rect.fit:

          # Rect.fit(Rect): return Rect
          # resize and move a rectangle with aspect ratio
          #
          # Returns a new rectangle that is moved and resized to fit another.
          # The aspect ratio of the original Rect is preserved, so the new
          # rectangle may be smaller than the target in either width or height.

        r = Rect(10, 10, 30, 30)

        r2 = Rect(30, 30, 15, 10)

        f = r.fit(r2)
        self.assertTrue(r2.contains(f))

        f2 = r2.fit(r)
        self.assertTrue(r.contains(f2))



    def test_copy(self):
        r = Rect(1, 2, 10, 20)
        c = r.copy()
        self.assertEqual(c, r)

class PGZeroRectText(unittest.TestCase):

    def test_constructor_from_tuple(self):
        "Build a Rect from a 4-item tuple"
        r = Rect((1, 2, 3, 4))
        self.assertEqual(r.x, 1)
        self.assertEqual(r.y, 2)
        self.assertEqual(r.w, 3)
        self.assertEqual(r.h, 4)

    def test_constructor_from_PGZeroRect(self):
        "Build a rect from another pgzero Rect"
        r = Rect(1, 2, 3, 4)
        r1 = Rect(r)
        self.assertEqual(r, r1)
        self.assertIsNot(r, r1)

    def test_constructor_from_PygameRect(self):
        "Build a rect from a pygame Rect"
        r = pygame.Rect(1, 2, 3, 4)
        r1 = Rect(r)
        self.assertEqual(r, r1)
        self.assertIsNot(r, r1)

    def test_constructor_from_rect_tuple(self):
        "Build a rect from an object with a rect attribute which is a tuple"
        class Obj: pass
        obj = Obj()
        obj.rect = 1, 2, 3, 4
        r = Rect(obj)
        self.assertEqual(r, Rect(1, 2, 3, 4))

    def test_constructor_from_rect_object(self):
        "Build a rect from an object with a rect attribute which is an object"
        class Obj: pass
        obj = Obj()
        obj.rect = Rect(1, 2, 3, 4)
        r = Rect(obj)
        self.assertEqual(r, Rect(1, 2, 3, 4))

    def test_constructor_from_rect_indirect_tuple(self):
        """Build a rect from an object with a rect attribute which
        is an object which has a rect attribute which is a tuple
        """
        class Obj: pass
        obj = Obj()
        obj1 = Obj()
        obj1.rect = Rect(1, 2, 3, 4)
        obj.rect = obj1
        r = Rect(obj)
        self.assertEqual(r, Rect(1, 2, 3, 4))

    def test_constructor_from_rect_callable(self):
        """Build a rect from an object with a rect attribute which
        is called and returns a tuple
        """
        class Obj:
            def rect(self):
                return 1, 2, 3, 4
        obj = Obj()
        r = Rect(obj)
        self.assertEqual(r, Rect(1, 2, 3, 4))

    def test_float_instance(self):
        "Create an instance with floating-point co-ordinates"
        r = Rect(1.2, 3.4, 5.6, 7.8)
        self.assertEqual(r.x, 1.2)
        self.assertEqual(r.y, 3.4)
        self.assertEqual(r.w, 5.6)
        self.assertEqual(r.h, 7.8)

    def test_float_instance_as_PygameRect(self):
        "Create a Pygame Rect from a pgzero floating-point Rect"
        r = Rect(1.2, 3.4, 5.6, 7.8)
        r2 = pygame.Rect(r)
        self.assertEqual([int(i) for i in r], list(r2))

    def test_float_center(self):
        r = Rect(0, 0, 5, 5)
        self.assertEqual(r.center, (2.5, 2.5))

    def test_float_inflated(self):
        r = Rect(0, 0, 5, 5)
        self.assertEqual(r.inflate(1.5, 1.5), (-0.75, -0.75, 6.5, 6.5))

    def test_contains_point(self):
        r = Rect(0, 0, 100, 100)
        self.assertTrue((50, 50) in r)
        self.assertFalse((150, 150) in r)

    def test_contains_rect(self):
        r = Rect(0, 0, 100, 100)
        self.assertTrue((20, 20, 50, 50) in r)
        self.assertFalse((20, 20, 100, 100) in r)

    def test_not_hashable(self):
        r1 = Rect(1, 2, 3, 4)
        r2 = Rect(r1)
        self.assertEqual(r1, r2)
        self.assertIsNot(r1, r2)
        with self.assertRaises(TypeError):
            self.assertEqual({r1}, {r2})

    def test_as_tuple(self):
        t = 1, 2, 3, 4
        r1 = Rect(t)
        self.assertEqual(tuple(r1), t)

    def test_collidepoint_tuple(self):
        r = Rect(0, 0, 1, 1)
        self.assertTrue(r.collidepoint((0.5, 0.5)))

if __name__ == '__main__':
    unittest.main()



import os.path
from setuptools import setup
import pgzero

path = os.path.join(os.path.dirname(__file__), 'README.rst')
with open(path, encoding='utf8') as f:
    LONG_DESCRIPTION = f.read()

install_requires = [
    'pygame~=2.0',
    'numpy',
]

extras_require = {
    ':python_version < "3.4"': ["enum34"],
}

setup(
    name='pgzero',
    version=pgzero.__version__,
    description="A zero-boilerplate 2D games framework",
    long_description=LONG_DESCRIPTION,
    author='Daniel Pope',
    author_email='mauve@mauveweb.co.uk',
    url='http://pypi.python.org/pypi/pgzero',
    packages=['pgzero'],
    include_package_data=True,
    py_modules=['pgzrun'],
    entry_points={
        'console_scripts': [
            'pgzrun = pgzero.runner:main'
        ]
    },
    install_requires=install_requires,
    extras_require=extras_require,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Education',
        'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Education',
        'Topic :: Games/Entertainment',
    ],
    test_suite='test'
)



#
# TODO: Add sound effects
#       Ignore clicks after displaying second card until
#       either hit or miss is reported.
#       Consider re-casting the data structure as a dict
#       with two-element tuple keys.
#       Configure window size according to COLS and ROWS
#

checkmark = Actor('checkmark')
steve = Actor('card_back', (50, 50))
steve.topleft = (0, 0)

import random
import time

COLS = 4
ROWS= 3
IMSIZE = 200
STATUS = []        # cells that have been clicked on
ignore = []        # cells that have been matches and are no longer in play

# Create two of each card image, then randomize before creating the board
START_IMAGES= [ "im"+str(i+1) for i in range(COLS*ROWS//2)]*2
random.shuffle(START_IMAGES)

STATUS=[]

board = []                    # initialize the board
for row in range(ROWS):
    new_row=[]
    for col in range(COLS):
        image_name = START_IMAGES.pop()
        temp=Actor(image_name, (col*IMSIZE, row*IMSIZE))
        temp.image_name = image_name # used to verify matches
        temp.topleft=(col*IMSIZE, row*IMSIZE)
        new_row.append(temp)
    board.append(new_row)

def draw():                    # draw the board when pygame-zero says to
    screen.clear()
    for row in range(ROWS):
        for col in range(COLS):
            if (row, col) in ignore:    # already matched
                checkmark.topleft = IMSIZE*col, IMSIZE*row
                checkmark.draw()
            elif (row, col) in STATUS:    # clicked this move: show face
                board[row][col].draw()
            else:                        # regular clickable card
                steve.topleft = IMSIZE*col, IMSIZE*row
                steve.draw()

def findTile(pos):
    y, x = pos
    result = x // IMSIZE , y // IMSIZE
    return result

def showTile():
    pass


def on_mouse_down(pos, button):
    if len(STATUS) == 2: # ignore until timeout redisplays
        return
    if pos in ignore: # has already been matched
        return
    if button == mouse.LEFT and (pos):
    # not sure why "and (pos)" - especially the parens!
        coords = findTile(pos)
        if coords not in STATUS:
            STATUS.append(coords) # now they are
            if len(STATUS) == 1:  # 1st click - turn not yet over
                pass
            elif len(STATUS) == 2: # 2nd click - check for match
                (x1, y1), (x2, y2) = STATUS # an "unpacking assignment"
                if board[x1][y1].image_name == board[x2][y2].image_name:
                    print("Success sound")
                    # add cards to list of non-clickable positions
                    for pos in STATUS:
                        ignore.append(pos)
                else:
                    print("Failure sound")
                clock.schedule_unique(next_turn, 2.0)

def next_turn():
    del STATUS[:]

